
/*
 * Copyright (c) Developers of JMST
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 25 August 2004	Jonee Ryan Ty		First documented history
 *
 */
 
import java.awt.Color;

import java.util.Vector;
import java.util.Enumeration;


public class Graph
{
	public Vector gEdges, gNodes;	// data structure containers
		 
	public Graph() {
		gEdges = new Vector();
		gNodes = new Vector();
	}
	
	/**
	 * To add an edge to this graph.
	 * @param the vertices of the edge
	 */	
	public void addEdge(GNode node1, GNode node2) {
		try {			
		// do not create edge when a line connecting two nodes already exist
			if(!node1.isConnectedToNode(node2)) {
					// may throw SameNodeException
				Edge newEdg = new Edge(node1, node2);	
										
				node1.addEdge(newEdg);		// add connection to edges
				node2.addEdge(newEdg);
						
				node1.addNode(node2);		// add connection to nodes
				node2.addNode(node1);
													
				gEdges.add(newEdg);			// add newEdg
			}	
		
		} catch(Edge.SameNodeException sne) {}	// do not create a new line 
	}
	
	public void addEdge(Edge e) {
		GNode node1 = e.node1, node2 = e.node2;
		
		node1.addEdge(e);		// add connection to edges
		node2.addEdge(e);
						
		node1.addNode(node2);		// add connection to nodes
		node2.addNode(node1);
													
		gEdges.add(e);			// add newEdg		
	}
	
	/**
	 * To remove an edge to this graph.
	 * @param rem the edge to remove
	 */	
	public void removeEdge(Edge rem) {
		
		rem.node1.removeEdge(rem);			// remove node links to this line
		rem.node2.removeEdge(rem);		
						
		rem.node1.removeNode(rem.node2); 	// remove connection between
		rem.node2.removeNode(rem.node1);				// nodes
						
		this.gEdges.remove(rem);			// remove line in vector
	}	

	/**
	 * To remove all edges of this graph.
	 */	 
	public void removeAllEdges() {	
	
		Enumeration enumNodes = gNodes.elements();
		GNode tmpNde;	
			
		while(enumNodes.hasMoreElements()) {
			tmpNde = (GNode)(enumNodes.nextElement());
			
			tmpNde.edges.removeAllElements();	// delete all "connections"
			tmpNde.edges.trimToSize();
			
			tmpNde.nodes.removeAllElements();
			tmpNde.nodes.trimToSize();
		}
		
		this.gEdges.removeAllElements();
		this.gEdges.trimToSize();
		
		System.gc();
	}			
				
	/**
	 * Add a node to this graph.
	 * @param xMo, yMo, pixel coordinates to add new node
	 * @param c Color we associate with this node
	 */	 
	public void addNode(int xMo, int yMo, Color c) {
		this.gNodes.add(new GNode(xMo, yMo, c));
	}
	
	public void addNode(GNode toAdd) {
		this.gNodes.add(toAdd);
	}	
	
	/**
	 * Removes a node to this graph.
	 * @param rem the node to be deleted
	 */	 
	public void removeNode(GNode rem) {		
		Enumeration edgesToRemove  = rem.edges.elements();
		
		Edge temp;
								
		while(edgesToRemove.hasMoreElements()) {	
							// remove also lines connected
			temp = (Edge)edgesToRemove.nextElement();
												// remove links to other nodes
			temp.otherNode(rem).removeNode(rem);		
			temp.otherNode(rem).removeEdge(temp);
				
			this.gEdges.remove(temp);			// remove lines
		}
			
		this.gNodes.remove(rem);				// remove node in vector 	
		
		this.gEdges.trimToSize();
		this.gNodes.trimToSize();
			
		System.gc();	
	}
	
	/**
	 * Removes all nodes and edges in the graph.
	 */	 
	public void removeAll() {	
		this.gNodes.removeAllElements();
		this.gEdges.removeAllElements();
	
		this.gEdges.trimToSize();
		this.gNodes.trimToSize();
		
		System.gc();
	}
	
	/**
	 * Creates edges that would connect all nodes.
	 */	
	public void connectAllNodes() {			// connects all nodes
	
		for(int a = 0; a < this.gNodes.size(); a++)			// permutates
			for(int b = 0; b < this.gNodes.size(); b++)
				this.addEdge((GNode)(this.gNodes.elementAt(a)), 
					(GNode)(this.gNodes.elementAt(b)));
	}
	
	/**
	 * Determines the common edge of two nodes.
	 * @return the common edge if it exists or null if  not.
	 * @param node1, node2, the nodes we wish to find the common edge of
	 */	
	public Edge commonEdgeOf(GNode node1, GNode node2) {
		Edge temp, returnEdg = null;
		
		for(int i = 0; i < gEdges.size(); i++) {
			temp = (Edge)gEdges.elementAt(i);
			
			if(temp.isIncidentTo(node1) && temp.isIncidentTo(node2)) {
				returnEdg = temp;
				i = gEdges.size() + 1;					// to terminate loop
			}
		}
		
		return returnEdg;
	}
				
	/**
	 * Arranges the collection of edges of this graph in an increasing order.
	 */
	 	// Jonee's psuedo insertion sort
	public void arrangeEdges() {							
				// to arrange edges in order of increasing weights
		Vector arranged = new Vector();
		
		Enumeration enum = gEdges.elements();
			
		int tmpVal, a, insertAtIndex;
		Edge tmp;
		// add all elements one at a time into a new vector and arranging them
		while(enum.hasMoreElements()) {
			tmp = ((Edge)enum.nextElement());
			tmpVal = tmp.getWeight();
			
			a = insertAtIndex = 0;
			
			while(a < arranged.size()) {
				if(tmpVal <= ((Edge)arranged.elementAt(a)).getWeight()) {
					insertAtIndex = a;
					a = arranged.size() + 1;			// to terminate loop
				
				} else
					insertAtIndex++;
			
				a++;
			}
			
			arranged.add(insertAtIndex, tmp);
		}
		
		gEdges = arranged;	
	}
	
	/**
	 * Unmarks all nodes and edges
	 */	
	public void unincludeAll() {
		GNode tmp;
		for(int c = 0; c < gNodes.size(); c++) {
			tmp = (GNode)gNodes.elementAt(c);
			tmp.ndeIsIncluded = false;
			tmp.whereIncluded = null;
		}
			
		for(int d = 0; d < gEdges.size(); d++)
			((Edge)gEdges.elementAt(d)).edgIsIncluded = false;		
	}	
	
	/**
	 * returns the adjacency matrix of this graph
	 * @return boolean array representing the adjacency matrix
	 */	 
	 public boolean[][] getAdjMatrix() {
	 	int numOfNodes = this.gNodes.size();
	 	
	 	boolean adjMatrix[][] = new boolean[numOfNodes][numOfNodes];
	 	
	 	int horizLimit = 0;
	 	
	 	for(int g = 0; g < numOfNodes; g++) { // lower half triangular part
	 		for(int h = 0; h < horizLimit; h++)
	 			adjMatrix[g][h] = 
	 				((GNode)this.gNodes.elementAt(g)).isConnectedToNode(
	 				((GNode)this.gNodes.elementAt(h)));
	 		
	 		horizLimit++;
	 	}
	 	
	 	for(int j = 0; j < numOfNodes; j++)			// diagonal part
	 		adjMatrix[j][j] = false;
	 	
	 	int curHorizIndx = 1;
	 	
	 	for(int k = 0; k < numOfNodes; k++) {
	 		for(int l = curHorizIndx; l < numOfNodes; l++)
	 			adjMatrix[k][l] = adjMatrix[l][k];
	 			
	 		curHorizIndx++;
	 	}	
	
		return adjMatrix;
	}
	 
	 /**
	 * returns the incident matrix of this graph
	 * @return boolean array representing the incident matrix
	 */	 
	 public boolean[][] getIncMatrix() {
	 	int numOfNodes = this.gNodes.size(),
			numOfEdges = this.gEdges.size();
			
	 	boolean incMatrix[][] = new boolean[numOfNodes][numOfEdges];
				
		for(int m = 0; m < numOfNodes; m++) {
			for(int n = 0; n < numOfEdges; n++)	{
				incMatrix[m][n] = 
					((Edge)this.gEdges.elementAt(n)).isIncidentTo(
					(GNode)this.gNodes.elementAt(m));		
			}
		}
		
		return incMatrix;
	}
	
} //##30
